from dg_sdk.module.pay.scan_payment import ScanPayment
from dg_sdk.module.pay.credit_card_info import CreditCardInfo
from dg_sdk.module.pay.bank_card import BankCard
from dg_sdk.module.pay.acct_payment import AcctPayment
from dg_sdk.module.pay.card import Card
from dg_sdk.module.pay.cert import Cert
from dg_sdk.module.pay.delaytrans import Delaytrans
from dg_sdk.module.pay.drawcash import Drawcash
from dg_sdk.module.pay.online_payment import OnlinePayment
from dg_sdk.module.pay.surrogate import Surrogate



from dg_sdk.module.request_tools import request_post
from dg_sdk.module.merchant.merchant_api_urls import *
from dg_sdk.dg_client import DGClient
from dg_sdk.module.merchant.merchant_info import MerchantInfo
from dg_sdk.module.merchant.legal_info import LegalInfo
from dg_sdk.module.merchant.bussiness_lic_info import BussinessLicInfo
from dg_sdk.module.merchant.settle_config_info import SettleConfigInfo
from dg_sdk.module.merchant.cash_config_info import CashConfigInfo
from dg_sdk.module.merchant.mer_card_info import MerCardInfo
from dg_sdk.module.merchant.corp_info import CorpInfo
from typing import List
import os


class Merchant(object):
    """
    商户进件相关类，包含以下接口
    企业类型商户进件
    个体户类型商户进件
    基本信息修改
    详细信息查询
    新增总部
    修改总部
    总部商户绑定&解除
    查询账户信息
    商户业务开通
    商户业务开通修改
    申请单状态查询
    商户图片资料上传
    商户分账配置
    商户分账配置查询
    商户分期配置
    商户分期配置详情查询
    活动报名，支持微信
    """

    @classmethod
    def create_enterprise(cls, upper_huifu_id,
                          merchant_info: MerchantInfo,
                          card_info: MerCardInfo,
                          lic_info: BussinessLicInfo,
                          legal_info: LegalInfo,
                          settle_info: SettleConfigInfo = None,
                          cash_config: List[CashConfigInfo] = None,
                          settle_agree_pic="",
                          **kwargs):
        """
        企业类型商户进件
        :param upper_huifu_id: 渠道商汇付Id
        :param merchant_info: 商户经营信息
        :param lic_info: 营业执照信息
        :param legal_info: 法人信息
        :param card_info: 卡信息
        :param settle_info: 结算配置
        :param cash_config:取现配置列表
        :param settle_agree_pic: D1结算协议图片文件路径
        :param kwargs: 额外参数
        :return:
        """

        required_params = {
            "upper_huifu_id": upper_huifu_id,
            "product_id": DGClient.mer_config.product_id
        }
        if merchant_info:
            required_params.update(merchant_info.obj_to_dict())
        if lic_info:
            required_params.update(lic_info.obj_to_dict())
        if legal_info:
            required_params.update(legal_info.obj_to_dict())
        if card_info:
            required_params["card_info"] = card_info.obj_to_dict()
        if settle_info:
            required_params["settle_config"] = settle_info.obj_to_dict()
        if cash_config:
            configs = []
            for config in cash_config:
                configs.append(config.obj_to_dict())
            required_params["cash_config"] = configs

        file = None
        if not settle_agree_pic:
            settle_agree_pic = kwargs.get('settle_agree_pic')
            if settle_agree_pic:
                kwargs.pop('settle_agree_pic')

        if settle_agree_pic:
            file = {'settle_agree_pic': (
                os.path.basename(settle_agree_pic), open(settle_agree_pic, 'rb'), 'application/octet-stream')}

        required_params.update(kwargs)
        return request_post(ent_mer_reg, required_params, file)

    @classmethod
    def create_individual(cls, upper_huifu_id,
                          merchant_info: MerchantInfo,
                          card_info: MerCardInfo,
                          settle_info: SettleConfigInfo = None,
                          cash_config: List[CashConfigInfo] = None,
                          settle_agree_pic="",
                          short_name="",
                          **kwargs):
        """
        个人类型商户进件
        :param upper_huifu_id: 渠道商汇付Id
        :param merchant_info: 商户经营信息
        :param card_info: 卡信息
        :param settle_info: 结算配置
        :param cash_config:取现配置列表
        :param settle_agree_pic: D1结算协议图片文件路径
        :param short_name: 商户简称
        :param kwargs: 额外参数
        :return:
        """

        required_params = {
            "upper_huifu_id": upper_huifu_id,
            "product_id": DGClient.mer_config.product_id,
            "short_name": short_name,
        }
        if merchant_info:
            required_params.update(merchant_info.obj_to_dict())
        if card_info:
            required_params["card_info"] = card_info.obj_to_dict()
        if settle_info:
            required_params["settle_config"] = settle_info.obj_to_dict()
        if cash_config:
            configs = []
            for config in cash_config:
                configs.append(config.obj_to_dict())
            required_params["cash_config"] = configs

        file = None
        if not settle_agree_pic:
            settle_agree_pic = kwargs.get('settle_agree_pic')
            if settle_agree_pic:
                kwargs.pop('settle_agree_pic')

        if settle_agree_pic:
            file = {'settle_agree_pic': (
                os.path.basename(settle_agree_pic), open(settle_agree_pic, 'rb'), 'application/octet-stream')}

        required_params.update(kwargs)
        return request_post(ent_mer_reg, required_params, file)

    @classmethod
    def modify(cls, upper_huifu_id,
               merchant_info: MerchantInfo = None,
               lic_info: BussinessLicInfo = None,
               legal_info: LegalInfo = None,
               card_info: MerCardInfo = None,
               settle_info: SettleConfigInfo = None,
               cash_config: List[CashConfigInfo] = None,
               settle_agree_pic="",
               **kwargs):
        """
        修改商户基本信息
        :param upper_huifu_id: 渠道商汇付Id
        :param merchant_info: 商户经营信息
        :param lic_info: 营业执照信息
        :param legal_info: 法人信息
        :param card_info: 卡信息
        :param settle_info: 结算配置
        :param cash_config:取现配置列表
        :param settle_agree_pic: D1结算协议图片文件路径
        :param kwargs: 额外参数
        :return:
        """

        required_params = {
            "upper_huifu_id": upper_huifu_id,
            "product_id": DGClient.mer_config.product_id
        }
        if merchant_info:
            required_params.update(merchant_info.obj_to_dict())
        if lic_info:
            required_params.update(lic_info.obj_to_dict())
        if legal_info:
            required_params.update(legal_info.obj_to_dict())
        if card_info:
            required_params["card_info"] = card_info.obj_to_dict()
        if settle_info:
            required_params["settle_config"] = settle_info.obj_to_dict()
        if cash_config:
            configs = []
            for config in cash_config:
                configs.append(config.obj_to_dict())
            required_params["cash_config"] = configs

        file = None
        if not settle_agree_pic:
            settle_agree_pic = kwargs.get('settle_agree_pic')
            if settle_agree_pic:
                kwargs.pop('settle_agree_pic')

        if settle_agree_pic:
            file = {'settle_agree_pic': (
                os.path.basename(settle_agree_pic), open(settle_agree_pic, 'rb'), 'application/octet-stream')}

        required_params.update(kwargs)
        return request_post(mer_modify, required_params, files=file)

    @classmethod
    def query_merch_info(cls, **kwargs):
        """
        查询商户详细信息
        :return:
        """

        required_params = {
            "product_id": DGClient.mer_config.product_id
        }
        required_params.update(kwargs)
        return request_post(query_merch_info, required_params)

    @classmethod
    def add_headquarters(cls,
                         name,
                         contact_name,
                         contact_mobile_no,
                         contact_cert_no,
                         login_name="",
                         login_mobile_no="",
                         add_corp_info: CorpInfo = None,
                         bd_login_user_id="",
                         mer_huifu_id="",
                         **kwargs):
        """
        新增总部
        :param name: 总部名称
        :param contact_name: 联系人姓名
        :param contact_mobile_no: 联系人手机号
        :param contact_cert_no:  联系人身份证号码
        :param login_name: 管理员账号，有值，必须全网唯一；为空，不生成管理员账号
        :param login_mobile_no: 管理员手机号
        :param add_corp_info: 企业信息
        :param bd_login_user_id: 业务经理userId
        :param mer_huifu_id: 商户汇付Id
        :param kwargs:
        :return:
        """

        required_params = {
            "product_id": DGClient.mer_config.product_id,
            "name": name,
            "contact_name": contact_name,
            "contact_mobile_no": contact_mobile_no,
            "contact_cert_no": contact_cert_no,
            "login_name": login_name,
            "login_mobile_no": login_mobile_no,
            "bd_login_user_id": bd_login_user_id,
            "mer_huifu_id": mer_huifu_id,
        }
        if add_corp_info:
            required_params["add_corp_info"] = add_corp_info.obj_to_dict()

        required_params.update(kwargs)
        return request_post(add_chains_corp_info, required_params)

    @classmethod
    def modify_headquarters(cls,
                            name="",
                            contact_name="",
                            contact_mobile_no="",
                            contact_cert_no="",
                            edit_corp_info: CorpInfo = None,
                            mer_huifu_id="",
                            **kwargs):
        """
        修改总部
        :param name: 总部名称
        :param contact_name: 联系人姓名
        :param contact_mobile_no: 联系人手机号
        :param contact_cert_no:  联系人身份证号码
        :param edit_corp_info: 企业信息
        :param mer_huifu_id: 商户汇付Id
        :param kwargs:
        :return:
        """

        required_params = {
            "product_id": DGClient.mer_config.product_id,
            "name": name,
            "contact_name": contact_name,
            "contact_mobile_no": contact_mobile_no,
            "contact_cert_no": contact_cert_no,
            "mer_huifu_id": mer_huifu_id,
        }
        if edit_corp_info:
            required_params["edit_corp_info"] = edit_corp_info.obj_to_dict()

        required_params.update(kwargs)
        return request_post(modify_chains_corp_info, required_params)

    @classmethod
    def bind_headquarters(cls, chains_id, state, mer_type, **kwargs):
        """
        总部商户绑定&解除
        :param chains_id: 连锁编号,每个总部下最多添加50个商户
        :param state: 状态，1绑定，0 解除
        :param mer_type: 商户类型，0 自营，1 加盟
        :return:
        """

        required_params = {
            "product_id": DGClient.mer_config.product_id,
            "chains_id": chains_id,
            "state": state,
            "mer_type": mer_type,
        }
        required_params.update(kwargs)
        return request_post(bind_chains_mer, required_params)

    @classmethod
    def query_acct_info(cls, **kwargs):

        #TODO
        required_params = {
            "product_id": DGClient.mer_config.product_id
        }
        required_params.update(kwargs)
        return request_post(query_mer_split_config, required_params)



    @classmethod
    def reg_busi_info(cls, **kwargs):
        """
        商户业务开通
        :param kwargs:
        :return:
        """

        required_params = {
            "product_id": DGClient.mer_config.product_id
        }
        required_params.update(kwargs)
        return request_post(query_mer_split_config, required_params)


    @classmethod
    def query_split_config(cls, **kwargs):
        """
        查询商户分账配置信息
        :return:
        """

        required_params = {
            "product_id": DGClient.mer_config.product_id
        }
        required_params.update(kwargs)
        return request_post(query_mer_split_config, required_params)



from dg_sdk.module.request_tools import request_post
from dg_sdk.module.pay.pay_api_urls import online_payment_query, online_payment_refund, \
    online_payment_refund_query, union_app_pay, wap_pay


class OnlinePayment(object):
    """
    线上交易相关接口，银联APP支付，网银支付，手机网页支付，线上交易退款，线上交易退款查询，线上交易交易查询
    """
    # TODO 手机网页支付

    @classmethod
    def union_app_create(cls, trans_amt, **kwargs):
        """
        创建聚合正扫订单
        :param trans_amt: 交易金额，单位为元，（例如：100.00）
        :param kwargs:  非必填额外参数
        :return: 支付对象
        """

        required_params = {
            "trans_amt": trans_amt,
        }

        required_params.update(kwargs)
        return request_post(union_app_pay, required_params)

    @classmethod
    def wap_page(cls, bank_card_no, front_url, trans_amt, **kwargs):
        """
        手机网页支付
        :param bank_card_no:
        :param front_url:
        :param trans_amt:
        :param kwargs:
        :return:
        """

        required_params = {
            "trans_amt": trans_amt,
            "bank_card_no": bank_card_no,
            "front_url": front_url,
        }

        required_params.update(kwargs)
        return request_post(wap_pay, required_params)

    @classmethod
    def query(cls, org_req_date, *, org_req_seq_id="", org_hf_seq_id="", pay_type="", **kwargs):
        """
        线上交易查询接口
        :param org_req_seq_id: 原始请求流水号
        :param org_req_date: 原始订单请求时间
        :param pay_type: 原交易支付类型, 快捷支付传quick_pay
        :param org_hf_seq_id: 交易返回的全局流水号
        :param kwargs: 非必填额外参数
        :return: 返回报文
        """
        required_params = {
            "org_req_date": org_req_date,
            "org_req_seq_id": org_req_seq_id,
            "org_hf_seq_id": org_hf_seq_id,
            "pay_type": pay_type
        }

        required_params.update(kwargs)

        return request_post(online_payment_query, required_params)

    @classmethod
    def refund(cls, ord_amt, org_req_date, *, org_req_seq_id="", org_hf_seq_id="", **kwargs):
        """
        线上退款接口
        :param ord_amt: 退款金额
        :param org_req_seq_id: 原始请求流水号
        :param org_req_date: 原始订单请求时间
        :param org_hf_seq_id: 交易返回的全局流水号
        :param kwargs: 非必填额外参数
        :return: 返回报文
        """

        required_params = {
            "ord_amt": ord_amt,
            "org_req_date": org_req_date,
            "org_req_seq_id": org_req_seq_id,
            "org_hf_seq_id": org_hf_seq_id,
        }

        required_params.update(kwargs)

        return request_post(online_payment_refund, required_params)

    @classmethod
    def refund_query(cls, org_req_date, *, org_req_seq_id="", org_hf_seq_id="", **kwargs):
        """
        线上退款查询接口
        :param org_req_seq_id: 原始请求流水号
        :param org_req_date: 原始退款请求时间
        :param org_hf_seq_id: 交易返回的全局流水号
        :param kwargs: 非必填额外参数
        :return: 返回报文
        """
        required_params = {
            "org_req_date": org_req_date,
            "org_req_seq_id": org_req_seq_id,
            "org_hf_seq_id": org_hf_seq_id,
        }
        required_params.update(kwargs)
        return request_post(online_payment_refund_query, required_params)



from dg_sdk.module.mer_config import MerConfig
from dg_sdk.dg_client import DGClient
from dg_sdk.common_util import generate_mer_order_id
from dg_sdk.module.dg_tools import DGTools
from dg_sdk.module.pay import *
from dg_sdk.module.merchant import *





import unittest
import dg_sdk
from tests.conftest import *


class TestPayment(unittest.TestCase):

    def setUp(self):

        dg_sdk.DGClient.mer_config = dg_sdk.MerConfig(private_key, public_key, sys_id, product_id, huifu_id)

        print("setup")

    def tearDown(self):
        print("tearDown")

    def test_payment_create(self):
        result = dg_sdk.ScanPayment.create(trade_type="A_NATIVE", trans_amt="1.00", goods_desc="test")
        assert result["resp_code"] == "00000100"

    def test_payment_query(self):
        result = dg_sdk.ScanPayment.create(trade_type="A_NATIVE", trans_amt="1.00", goods_desc="test")

        result = dg_sdk.ScanPayment.query(org_req_date=result["req_date"], org_req_seq_id=result["req_seq_id"])

        assert result["resp_code"] == "00000000"

    def test_payment_close(self):
        result = dg_sdk.ScanPayment.create(trade_type="A_NATIVE", trans_amt="1.00", goods_desc="test")

        result = dg_sdk.ScanPayment.close(org_req_date=result["req_date"], org_req_seq_id=result["req_seq_id"])

        assert result["resp_code"] == "10000015"

    def test_payment_close_query(self):
        result = dg_sdk.ScanPayment.create(trade_type="A_NATIVE", trans_amt="1.00", goods_desc="test")
        result = dg_sdk.ScanPayment.close(org_req_date=result["req_date"], org_req_seq_id=result["req_seq_id"])
        result = dg_sdk.ScanPayment.close_query(org_req_date=result["req_date"], org_req_seq_id=result["req_seq_id"])

        assert result["resp_code"] == "20000004"

    def test_payment_refund(self):
        result = dg_sdk.ScanPayment.create(trade_type="A_NATIVE", trans_amt="1.00", goods_desc="test")

        result = dg_sdk.ScanPayment.refund(ord_amt="0.01", org_req_date=result["req_date"],
                                           org_req_seq_id=result["req_seq_id"])

        assert result["resp_code"] == "10000001"

    def test_payment_refund_query(self):
        result = dg_sdk.ScanPayment.create(trade_type="A_NATIVE", trans_amt="1.00", goods_desc="test")

        result = dg_sdk.ScanPayment.refund(ord_amt="0.01", org_req_date=result["req_date"],
                                           org_req_seq_id=result["req_seq_id"])
        result = dg_sdk.ScanPayment.refund_query(org_req_date=result["req_date"],
                                                 org_req_seq_id=result["req_seq_id"])

        assert result["resp_code"] == "20000004"




ent_mer_reg="/ssproxy/entMerReg"
indv_mer_reg="/ssproxy/indvMerReg"
mer_modify="/ssproxy/merModify"
query_merch_info="/ssproxy/queryMerchInfo"
add_chains_corp_info="/ssproxy/addChainsCorpInfo"
modify_chains_corp_info="/ssproxy/modifyChainsCorpInfo"
bind_chains_mer="/ssproxy/bindChainsMer"
query_acct_info_list="/ssproxy/queryAcctInfoList"
reg_mer_busi_info="/ssproxy/regMerBusiInfo"
modify_mer_busi_info="/ssproxy/modifyMerBusiInfo"
query_apply_status="/ssproxy/queryApplyStatus"
mer_file_upload="/ssproxy/file/upload"
add_mer_split_config="/ssproxy/addMerSplitConfig"
query_mer_split_config="/ssproxy/queryMerSplitConfig"
config_fq_mer_free="/ssproxy/configFqMerFee"
query_fq_mer_free_detail="/ssproxy/queryFqMerFeeDetail"
reg_activities="/ssproxy/regActivities"


import unittest
from tests.conftest import *


class TestAcctPayment(unittest.TestCase):

    def setUp(self):
        dg_sdk.DGClient.mer_config = dg_sdk.MerConfig(private_key, public_key, sys_id, product_id, huifu_id)

        print("setup")

    def tearDown(self):
        print("tearDown")

    def test_drawcash_create(self):
        result = dg_sdk.Drawcash.create(cash_amt="0.01",
                                           token_no="121231312",
                                           into_acct_date_type="D1")

        assert result["resp_code"] == "20000007"

    def test_drawcash_query(self):

        result = dg_sdk.Drawcash.query("20211123", org_req_seq_id="dsfasdfs")

        assert result["resp_code"] == "20000004"






from dg_sdk.module.merchant.merchant_info import MerchantInfo
from dg_sdk.module.merchant.legal_info import LegalInfo
from dg_sdk.module.merchant.bussiness_lic_info import BussinessLicInfo
from dg_sdk.module.merchant.mer_card_info import MerCardInfo

from dg_sdk.module.merchant.merchant import Merchant


import unittest
from tests.conftest import *


class TestAcctPayment(unittest.TestCase):

    def setUp(self):
        dg_sdk.DGClient.mer_config = dg_sdk.MerConfig(private_key, public_key, sys_id, product_id, huifu_id)

        print("setup")

    def tearDown(self):
        print("tearDown")



import unittest
import dg_sdk
from tests.conftest import *


class TestMerchant(unittest.TestCase):

    def setUp(self):
        dg_sdk.DGClient.mer_config = dg_sdk.MerConfig(private_key, public_key, sys_id, product_id, huifu_id)

        print("setup")

    def tearDown(self):
        print("tearDown")

    def test_query_merch_info(self):
        result = dg_sdk.Merchant.query_merch_info()
        assert result["sub_resp_code"] == "00000000"

    def test_query_split_config(self):
        result = dg_sdk.Merchant.query_split_config()
        assert result["sub_resp_code"] == "00000000"

    def test_modify(self):
        result = dg_sdk.Merchant.modify(upper_huifu_id=upper_huifu_id, settle_agree_pic="./test1.zip")

        assert result["sub_resp_code"] == "00000000"



from dg_sdk.core.rsa_utils import rsa_design, rsa_long_encrypt
from dg_sdk.module.request_tools import request_post
from dg_sdk.dg_client import DGClient
from dg_sdk.module.pay.pay_api_urls import union_user_id


class DGTools(object):

    @classmethod
    def verify_sign(cls, data, sign, pub_key=""):
        """
        校验返回报文签名
        :param data: 返回data
        :param sign:  返回签名
        :param pub_key: 公钥，默认使用SDK初始化时的公钥
        :return: 是否通过校验
        """
        if not pub_key:
            pub_key = DGClient.mer_config.public_key
        return rsa_design(sign, data, pub_key)

    @classmethod
    def union_user_id(cls, auth_code, app_up_identifier='CloudPay', **kwargs):
        """
        获取银联用户标识
        :param app_up_identifier: 银联支付标识,浏览器agent
        :param auth_code: 用户授权码
        :param kwargs: 非必填额外参数
        :return: 银联用户标识
        """
        required_params = {
            "user_auth_code": auth_code,
            "app_up_identifier": app_up_identifier
        }

        required_params.update(kwargs)
        return request_post(union_user_id, required_params)

    @classmethod
    def encrypt_with_public_key(cls, orignal_str, public_key=""):
        """
        通过RSA 公钥加密敏感信息
        :param orignal_str: 原始字符串
        :param public_key: 公钥，不传使用商户配置公钥
        :return: 密文
        """
        if not public_key:
            public_key = DGClient.mer_config.public_key
        return rsa_long_encrypt(orignal_str, public_key)

    @classmethod
    def request_post(cls, url, request_params, files=None):
        """
        网络请求通用方法，根据请求的URL 地址判断是V1还是V2版本接口
        :param url: 请求地址
        :param request_params: 请求参数
        :param files: 附带文件，可为空
        :return: 网络请求返回内容
        """
        return request_post(url=url, request_params=request_params, files=files)
